"""
Module to fix things removed in Django 3.2.

This is expected to cover most of the things listed in this section:
https://docs.djangoproject.com/en/stable/internals/deprecation/#deprecation-removed-in-3-2
"""
